# encoding: UTF-8
require_relative '../core'
require_relative '../dsl'

include Minireq::Core

# TODO redesign: move project structure related code to new module
module Minireq
  module Cli
    include Minireq::Core::Writer
    # directory structure of minireq project
    REQ = 'reqs'
    DOC = 'docs'
    TPT = 'tpts'
    ASSETS = 'docs/assets'
    PRJ = [REQ, DOC, TPT, ASSETS]

    def inreq(&block); Dir.chdir(REQ, &block); end
    def indoc(&block); Dir.chdir(DOC, &block); end
    def intpt(&block); Dir.chdir(TPT, &block); end
    def inassets(&block); Dir.chdir(ASSETS, &block); end

    def create_project
      PRJ.each {|f| Dir.mkdir(f) unless Dir.exist?(f)}
    end

    def create_requirement_source(id, params)
      req = Requirement.new(id, params)
      inreq { write_requirement_source(req) }
    end

    def get_document(load_requirements = true)
      doc = Minireq::Core::Document.new
      inreq{doc.load} if load_requirements
      doc
    end

    # @param id [String] requirement id
    # @return [Boolean] result, true if file created and false if req id not found
    def write_req(id)
      doc = get_document
      req = doc.find(id)
      return false unless req
      indoc { write_requirement(doc, req) }
      return true
    end

    def write_doc(file_name)
      doc = get_document
      indoc { write_document(doc, file_name) }
    end

  end
end
