# encoding: UTF-8
require 'forwardable'

module Minireq
  module Core

    class Node
      extend Forwardable

      attr_reader :id
      attr_reader :title
      attr_reader :attributes
      attr_accessor :parent
      attr_reader :items

      def_delegators :@items, :<<

      def initialize(id, options = {})
        @parent = nil
        @items = []
        @attributes = {}

        @id = id
        @title = options[:title]
        options.delete(:title)
        @attributes.merge!(options)
      end

      def visit(&block)
        return unless block_given?
        yield(self)
        @items.each { |n| n.visit(&block) }
      end

      def to_a
        a = []
        visit{|n| a << n}
        a
      end

      def get_ids
        to_a.inject([]){|a, i| a << i.id}
      end

      # TODO: 0 add #level method to support full output
      def level
        result = 1
        parent = @parent
        until parent.nil?
          result += 1
          parent = parent.parent
        end
        result
      end

    end
  end
end
