# encoding: UTF-8

require_relative 'node'
require_relative 'writer'

module Minireq
  module Core

    class Requirement < Node

      attr_reader :body

      def initialize(id, options = {})
        super(id, options)
        @body = options[:body]
        @attributes.delete(:body)
        @body ||= ''
      end

      def parent_id
        @attributes[:parent]
      end

      # @return [Array<String>] of body_link_macro in hierarchy
      def get_links
        to_a.inject([]) do |a, i|
          a.concat(i.body.scan(/\[\[([\w\.]*)\]\]/).flatten.uniq)
        end
      end

      # Sort items by attributes[child_order]
      def sort_childs
        return if @items.empty?
        return unless @attributes[:child_order]
        order = @attributes[:child_order].split(/ /)
        @items.sort! {|x, y| order.index(x.id) <=> order.index(y.id)}
      rescue Exception => e
        msg = ""
        msg << "An error occured during sort childs for [#{@id}].\n"
        msg << "- child_order: #{@attributes[:child_order]};\n"
        msg << "- child_ids  : #{@items.map{|i| i.id}.join(' ')}.\n"
        msg << "Fix the error and try again.\n"
        msg << e.message
        msg << "\n\n"
        puts msg
      end

      def user_attributes
        a = {}
        attributes.each{|k, v| a[k] = v unless system_attribute?(k)}
        a
      end

      def system_attributes
        a = {}
        attributes.each{|k, v| a[k] = v if system_attribute?(k)}
        a
      end

      private

        def system_attribute?(a)
          [:parent, :child_order, :suppress_id].each{|i| return true if a.eql?(i)}
          return false
        end

    end
  end
end
