# encoding: UTF-8

require 'pp'
require_relative 'excerpt'
require_relative 'requirement'

module Minireq
  module Core

    module Writer
      include Excerpt

      def write_requirement_source(req)
        content = "\# [#{req.id}]"
        content << " #{req.title}" unless req.title.nil?
        content << "\n"
        content << hash_to_excerpt(req.attributes) unless req.attributes.empty?
        content << "\n" + req.body if req.body
        File.write("#{req.id}.md", content)
      end

      def write_requirement(doc, req)
        File.open(req.id + '.md', 'w') do |file|
          req.visit {|r| file.write(requirement_to_text(doc, r))}
        end
      end

      def write_document(doc, file_name)
        File.open(file_name, 'w') do |file|
          doc.visit {|r| file.write(requirement_to_text(doc, r)) }
        end
      end

      def requirement_to_text(doc, req)
        text = [title(req)]
        text << hash_to_markdown_table(req.user_attributes) unless req.user_attributes.empty?
        text << replace_links(doc, req.body) unless req.body.empty?
        text.join("\n\n") + "\n\n"
      end

      def hash_to_markdown_table(hash)
        head = "Attribute | Value\n--------- | -----"
        body = hash.inject(''){|s, (k, v)| s << "\n#{k} | #{v}"}
        head + body
      end

      def title(node)
        result = ''
        result << "[#{node.id}] " unless node.attributes[:suppress_id]
        result << node.title unless node.title.nil?
        result << " {##{get_url(node.id)}}"
        '#' * node.level << " #{result}"
      end

      def replace_links(doc, text)
        result = String.new(text)
        links = result.scan(/\[\[([\w\.]*)\]\]/).flatten.uniq
        links.each do |link|
          result.gsub!("[[#{link}]]", make_link(doc, link))
        end
        result
      end

      def get_url(title)
        title.downcase
          .gsub(/[^A-Za-z0-9\s]/, ' ')
          .strip
          .gsub(/(\s){2,}/, ' ')
          .gsub(/\s/, '-')
      end

      # @return [String] markdown link to requirement header
      # @param [String] requirment idifier [id]
      def make_link(doc, id)
        req = doc.find(id)
        return "[#{id}](unknown_requirement_id)" unless req
        title = ""
        title << "[#{req.id}] " unless req.attributes[:suppress_id]
        title << req.title if req.title
        "[#{title}](##{get_url(req.id)})"
      end
    end

  end
end
