# encoding: UTF-8

require_relative 'dsl/fpa'
require_relative 'dsl/fpa_dsl'
require_relative 'dsl/pert'
require_relative 'dsl/pert_dsl'
require_relative 'dsl/trace_dsl'

module Minireq
  module Dsl

=begin
TODO translate function points to hours

    f - function point count.
    m - man-months of effort to be derived from the function point count.
    s - schedule months for optimal schedule.
    j - Jones's first-order estimate exponent.

Exponents for Jones's first-order estimate

Kind of software | Best in class | Average | Worst in class
:--------------- | ------------- | ------- | --------------
Systems     | 0.43 | 0.45 | 0.48
Business    | 0.41 | 0.43 | 0.46
Shrink-wrap | 0.39 | 0.42 | 0.45

Calculate schedule months from function points

Jones's first order estimate formula uses the exponent, j, from the above table to compute schedule months, s, from function points, f. Schedule months do not include the requirements analysis phase, because this must have been completed to get the design needed for the function point count.
    s = f**j
Calculate effort in man-months from function points

The following formula converts function points into total man-months.
    m = f**(3*j) / 27
=end

    # TODO cocomo factors for diffrent kind of projects
    # Calculate Effort Applied, Development Time and People Required
    # @return [Hash] of calculated parameters
    # @param e95 [Number] effort estimation
    # @param hours_per_month [Number] real working hours per month
    def simple_cocomo(e95, hours_per_month = 165 * 0.6)
      e = e95 * 4 / hours_per_month #E95 is 25% of summary effort
      t = 2.5 * e**(1.0/3.0)
      p = e / t
      {effort_applied: e, development_time: t, people_required: p}
    end

  end
end
