#encoding: UTF-8

module Minireq
  module Dsl
    module Fpa
      # Calculate unadjusted funtion points
      # TODO check input values and their cobinations
      def ufp(type:, det:, ret: 0, ftr: 0)
        complexity = case type
          when :ILF, :EIF
            case ret
            when 1;    det > 50 ? AVG : LOW
            when 2..5; det < 20 ? LOW : det > 50 ? HIGH : AVG
            else       det < 20 ? AVG : HIGH
            end
          when :EI
            case ftr
            when 0..1; det > 15 ? AVG  : LOW
            when 2;    det > 15 ? HIGH : det > 4 ? AVG : LOW
            else       det > 4  ? HIGH : AVG
            end
          when :EO, :EQ
            case ftr
            when 0..1; det > 19 ? AVG  : LOW
            when 2..3; det > 19 ? HIGH : det > 5 ? AVG : LOW
            else       det > 5 ? HIGH : AVG
            end
        end
        UFP_VALUES[type][complexity]
      end

      # @return [Number] Value Adjustment Factor
      # @param system_characteristic [Hash] vaule must be in range 0..5.
      # Hash keys don't bring anything in calculation
      # [:data_communications, :distributed_data_processing, :performance, :heavily_used_configuration, :transaction_rate, :online_data_entry, :enduser_efficiency, :online_update, :complex_processing, :reusability, :installation_ease, :operational_ease, :multiple_sites, :facilitate_change]
      # TODO check value in range?
      def vaf(system_characteristics)
        sum = system_characteristics.values.inject(0, :+)
        sum * 0.01 + 0.65
      end

      private

      POINT_TYPE = [:ILF, :EIF, :EI, :EO, :EQ]
      COMPLEXITY = [:LOW, :AVG, :HIGH]
      FACTOR     = [:DET, :RET, :FTR]

      UFP_VALUES = {
        :ILF => [7, 10, 15],
        :EIF => [5,  7, 10],
        :EI  => [3,  4,  6],
        :EO  => [4,  5,  7],
        :EQ  => [3,  4,  6]}

      LOW = 0
      AVG = 1
      HIGH= 2

    end
  end
end
