# encoding: UTF-8

require 'forwardable'
require_relative 'pert'

module Minireq
  module Dsl

    class PertDSL
      include Pert
      extend Forwardable

      # [Hash<Object>]
      attr_reader :items

      # @!macro [attach] def_delegators
      #   @!method $2
      #     Forwards to $1.
      #     @see Hash#$2
      def_delegators :@items, :[]
      def_delegators :@items, :include?
      def_delegators :@items, :size
      def_delegators :@items, :clear

      def initialize
        @items = {}
      end

      def estimate(&block)
        instance_eval(&block) if block_given?
      end

      # calculate E95%
      def calculate
        eff = @items.collect {|k, v| v[:effort]}
        err = @items.collect {|k, v| v[:rmserr]}
        calc_e95(eff, err)
      end

      # add new estimation
      def pert(object, o:, m:, p:)
        # TODO rescue this exception
        raise "An attempt of adding a duplicate '#{object}'" if @items[object]
        @items[object] = {o: o, m: m, p: p}.merge(calc_pert(o: o, m: m, p: p))
      end
    end

  end
end
