# encoding: UTF-8

require 'pp'
require 'forwardable'
require_relative '../core'

module Minireq
  module Dsl

=begin
      Add traceaility item
      Usage:
      ```ruby
      include Minireq::Data::Tracer
      trace do
        from 'id.1', :source, 'src/id.1.rb'
        from 'id.2', :source, 'src/id.2.rb'
        from 'id.1', :source, 'src/id.1.helper.rb'
        from 'id.1', :test, 'src/id.1.spec.rb'

        from 'id.2', :source, ['lib/id.1.rb', 'lib/id.1.helper.rb']
        from 'id.2', :test,   'spec/id.1.spec.rb'
        from 'id.2', :guide,  ['doc/user-guide', 'doc/admin-guide']
        pp Repository.instance.items
      end
      ```
=end
      # Traceability objects repository
      # TODO Traces load from folder, write to folder
      class TraceDSL
        extend Forwardable

        # [Array<Trace>]
        attr_reader :items

        # @!macro [attach] def_delegators
        #   @!method $2
        #     Forwards to $1.
        #     @see Hash#$2
        # def_delegator :@items, :[]
        # def_delegator :@items, :each
        def_delegator :@items, :size
        # def_delegator :@items, :include?,

        def initialize
          @items = {}
        end

        def trace(&block)
          instance_eval(&block) if block_given?
        end

        # DSL function from
        def from(object, key, value)
          @items[object] = {} unless @items[object]
          @items[object].merge!({key => []}) unless @items[object].include?(key)
          value = [value] unless value.is_a? Array
          @items[object][key].concat(value)
        end
      end
  end
end
